/* -*- c-file-style: "java"; indent-tabs-mode: nil -*- */

#include "config.h"

#include <assert.h>
#include <stdio.h>
#include <sys/select.h>
#include <signal.h>
#include <sys/file.h>
#include <sys/time.h>
#include <time.h>
#include <sys/stat.h>

#include "distcc.h"
#include "zeroconf.h"
#include "trace.h"
#include "exitcode.h"

/* Called when publishing of service data completes */
static sw_result publish_reply(sw_discovery discovery, sw_discovery_publish_status status, sw_discovery_oid oid, sw_opaque extra) {
    int *done = extra;
    *done = 1;

    if (status != SW_DISCOVERY_PUBLISH_STARTED && status != SW_DISCOVERY_PUBLISH_STOPPED)
        rs_log_crit("zeroconf publishing failed (%i)", status);

    return SW_OKAY;
}

/* register a distcc service in DNS-SD/mDNS with the given port and number of CPUs */
int dcc_zeroconf_register(sw_discovery *discovery, int port, int n_cpus) {
    sw_salt salt;
    sw_discovery_oid oid;
    int done = 0;
    char service[256], hn[256], t[16];
    sw_text_record txt;
    
    if (sw_discovery_init(discovery) != SW_OKAY) {
        rs_log_crit("sw_discovery_init() failed.\n");
        return EXIT_CONNECT_FAILED;
    }

    /* Prepare TXT RR */
    if (sw_text_record_init(&txt) != SW_OKAY) {
        rs_log_crit("sw_text_record_init() failed.\n");
        sw_discovery_fina(*discovery);
        return EXIT_CONNECT_FAILED;
    }

    if (sw_discovery_salt(*discovery, &salt) != SW_OKAY) {
        rs_log_crit("sw_discovery_salt() failed.\n");
        goto fail;
    }

    /* Prepare service name */
    gethostname(hn, sizeof(hn)-1);
    hn[sizeof(hn)-1] = 0;
    snprintf(service, sizeof(service), "distcc@%s", hn);

    /* Prepare TXT RR */
    snprintf(t, sizeof(t), "cpus=%i", n_cpus);
    sw_text_record_add_string(txt, t);

    /* Publish the info */
    if (sw_discovery_publish(
            *discovery,
            0,
            service,
            "_distcc._tcp",
            NULL,
            NULL,
            (sw_port) port,
            sw_text_record_bytes(txt),
            sw_text_record_len(txt),
            publish_reply,
            &done,
            &oid) != SW_OKAY) {
        
        rs_log_crit("sw_discovery_publish() failed.\n");
        goto fail;
    }

    /* Wait until the publishing is finished */
    while (!done) {
        if (sw_salt_step(salt, NULL) != SW_OKAY) {
            rs_log_crit("sw_salt() failed.\n");
            goto fail;
        }
    
    }

    rs_log_info("zeroconf publishing complete");

    sw_text_record_fina(txt); 
    
    return 0;
    
 fail:
    
    sw_discovery_fina(*discovery);
    sw_text_record_fina(txt);
    return EXIT_CONNECT_FAILED;
}

/* Unregister this server from DNS-SD/mDNS */
int dcc_zeroconf_unregister(sw_discovery discovery) {

    sw_discovery_fina(discovery);
    return 0;
}
